/*
 * Hanashizume no Matsuri for X
 * Copyright (c) 2000, 2001 TF <tf@denpa.org>
 * All rights reserverd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: scenario.c,v 1.61 2001/03/12 15:36:21 tf Exp $
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "hana.h"
#include "flags.h"

extern ARCHIVE cg, msg;

typedef struct {
  unsigned char *block[16];
  int size[16];
} SCRIPT;

static int next_state(int);
static int next_block(int);
static int next_block_altn(int);
static int next_block_ending(int);
static int next_script(const char *, int);
static int next_script_altn(const char *, int);
static int next_script_offset(const char *, int, int);
static int next_script_ending(const char *, int);
static int next_script_omake(const char *, int);
static int goto_title(void);

SCRIPT script;

int get_script(ARCHIVE mes, const char *id)
{
  RESINFO *r;
  int i, b1, b2;

  r = search_rlist(mes.rlist, id);
  if (r == NULL) {
    fprintf(stderr, "can't find the message \"%s\".\n", id);
    quit_proc(top, NULL, NULL);
  }
  fseek(mes.fd, r->offset, SEEK_SET);

  for (i = 0; i < 16; i++) {
    b1 = fgetc(mes.fd);
    b2 = fgetc(mes.fd);
    script.size[i] = (b1 | (b2 << 8)) * 2;
    /* fprintf(stderr, "block %d (size = %d):\n", i, script.size[i]); */
    if (script.size[i] == 0) {
      script.block[i] = NULL;
    } else {
      script.block[i] = (unsigned char *)calloc(script.size[i], sizeof(unsigned char));
      fread(script.block[i], 1, script.size[i], mes.fd);
    }
  }

  return 0;
}

void free_script()
{
  int i;

  for (i = 0; i < 16; i++) {
    if (script.block[i] != NULL) {
      free(script.block[i]);
      script.block[i] = NULL;
    }
  }
}

void init_scenario()
{
  scn_stat.newpage = False;
  scn_stat.alternative = False;
  scn_stat.alt_val = -1;

  scn_stat.flag_megumi = 0;
  scn_stat.flag_yue = 0;
  scn_stat.flag_ruria = 0;
  scn_stat.flag_sarasa = 0;
  scn_stat.flag_hinata = 0;
  scn_stat.flag_himiko = 0;
  scn_stat.flag_mao = 0;
  scn_stat.flag_kurumi = 0;
  scn_stat.flag_minako = 0;

  strncpy(scn_stat.script, "0x1d", NAMELEN);
  scn_stat.block = 0;
  scn_stat.offset = 0;

  scn_stat.state = STAT_INIT;
  scn_stat.init_state = STAT_OPENING;

  strncpy(scn_stat.pe_script, "0x7", NAMELEN);
  scn_stat.pe_block = 5;
  scn_stat.pe_offset = 0;
}

void do_scenario(ARCHIVE msg, COMMAND *cmd)
{
  switch (scn_stat.state) {
  case STAT_INIT:
    get_script(msg, scn_stat.script);
    scn_stat.state = scn_stat.init_state;
    break;
  case STAT_OMAKE:
    cmd->type = SET_CG_OMAKE;
    scn_stat.state = STAT_PROC;
    break;
  case STAT_PROC:
  case STAT_OPENING:
  case STAT_OPENING_FAST:
  case STAT_ENDING:
  case STAT_ENDING_FAST:
    parse_block(scn_stat.block, cmd);
    if (scn_stat.offset >= script.size[scn_stat.block]) {
      fprintf(stderr, "#\n");
      fprintf(stderr, "# scene %s-%d done\n", scn_stat.script, scn_stat.block);
      fprintf(stderr, "#\n");
      scn_stat.state = next_state(-1);
      fprintf(stderr, "#\n");
      fprintf(stderr, "# next scene is %s-%d\n", scn_stat.script, scn_stat.block);
      fprintf(stderr, "#\n");
    }
    break;
  case STAT_ALTN:
    if (scn_stat.alternative == True) {
      scn_stat.state = next_state(scn_stat.alt_val);
      scn_stat.alt_val = -1;
      scn_stat.newpage = True;
      scn_stat.alternative = False;
    } else {
      proc_alternative(scn_stat.block);
      scn_stat.alternative = True;
    }
    break;
  case STAT_WAIT:
    // fprintf(stderr, "terminate the scenario.\n");
    return;
  default:
    fprintf(stderr, "Unknown state[%d].\n", scn_stat.state);
    quit_proc(top, NULL, NULL);
  }
}

void parse_block(int bid, COMMAND *cmd)
{
  int i, j, k, c, ucs, ucs2;
  char *p;
  char tmpbuf[BUFSIZ];

  i = scn_stat.offset;

  ucs = (script.block[bid][i + 1] << 8) | script.block[bid][i];
  i += 2;
  scn_stat.offset += 2;
  if (ucs == '\\') { /* a backslash disable a next newline */
    ucs = (script.block[bid][i + 1] << 8) | script.block[bid][i];
    if (ucs == 0x0a) {
      scn_stat.offset += 2;
      cmd->type = NONE;
      return;
    } else {
      /* 0x29:6 includes such a pattern */
      scn_stat.offset += 2;
      fprintf(stderr, "No newline follows backslash!\n");
      cmd->type = NONE;
      return;
    }
  } else if (ucs == 0x0a) { /* newline */
    cmd->type = LINE_FEED;
    return;
  } else if (ucs < 0x100) { /* ASCII string (command) */
    if (ucs == 'p') {
      cmd->type = WAIT_CLICK;
      return;
    } else if (ucs == 'e') {
      if (i + 1 < script.size[bid]) {
	ucs2 = (script.block[bid][i + 1] << 8) | script.block[bid][i];
      } else {
	ucs2 = -1;
      }
      if (ucs2 == 's') {
	/* 'es' command */
	goto arg_cmd;
      } else {
	cmd->type = NEW_PAGE;
	return;
      }
    } else if (ucs == 'n') {
      cmd->type = NONE;
      fprintf(stderr, "Unknown command[n]\n");
      return;
    } else if (ucs == 'f') {
      cmd->type = NONE;
      fprintf(stderr, "Unknown command[f]\n");
      return;
    } else if (ucs == 'h') {
      cmd->type = HIGHLIGHT;
      fprintf(stderr, "highlight[n]\n");
      return;
    }
  arg_cmd:

    // fprintf(stderr, "command: %c", ucs);
    c = ucs;

    for (j = 0; i < script.size[bid]; j++) {
      ucs = (script.block[bid][i + 1] << 8) | script.block[bid][i];
      if (ucs > 0x100 || ucs == 0x0a || ucs == '\\') {
	break;
      }
      scn_stat.offset += 2;
      i += 2;
      cmd->buf[j] = ucs;
    }
    cmd->buf[j] = 0;
    // fprintf(stderr, "[%s]\n", cmd->buf);
    switch (c) {
    case 'a':
      cmd->type = SET_CG_FADE;
      fprintf(stderr, "set CG: a[%s]\n", cmd->buf);
      p = cmd->buf;
      while (*p != '/') {
	/* Some scripts include lowercase filename
	 * but all resource names are uppercase.
	 */
	*p = toupper(*p);
	p++;
      }
      *p++ = '\0';
      while (*p != '\0') {
	if (*p == 'm') {
	  // fprintf(stderr, "\tgrayscale\n");
	  cmd->attr.grayscale.enable = True;
	  cmd->attr.grayscale.r = 256;
	  cmd->attr.grayscale.g = 256;
	  cmd->attr.grayscale.b = 256;
	  p++;
	  j = 0;
	  while (isdigit(*p) || *p == ',') {
	    tmpbuf[j++] = *p++;
	  }
	  tmpbuf[j] = '\0';
	  if (j > 5) {
	    sscanf(tmpbuf, "%d,%d,%d",
		   &cmd->attr.grayscale.r,
		   &cmd->attr.grayscale.g,
		   &cmd->attr.grayscale.b);
	    fprintf(stderr, "color effect[%d, %d, %d]\n",
		    cmd->attr.grayscale.r,
		    cmd->attr.grayscale.g,
		    cmd->attr.grayscale.b);
	  }
	} else if (*p == 'f') {
	  // fprintf(stderr, "\tfade\n");
	  p++;
	  j = 0;
	  while (isdigit(*p)) {
	    tmpbuf[j++] = *p++;
	  }
	  tmpbuf[j] = '\0';
	  cmd->attr.fade = True;
	  sscanf(tmpbuf, "%d", &(cmd->attr.fadetime));
	} else {
	  /* unknown token is dealed as the end of 'a' command */
	  // fprintf(stderr, "unknown token [%c]\n", *p);
	  while (*p++ != '\0') {
	    scn_stat.offset -= 2; /* push back the scenario */
	  }
	  break;
	}
      }
      break;
    case 'e': /* in case 'es', reach here */
      cmd->type = NONE;
      sscanf(cmd->buf + 1, "%d", &(cmd->arg));
      fprintf(stderr, "unknown: e%c[%d]\n", cmd->buf[0], cmd->arg);
      /* 'es' command takes int argument */
      p = &cmd->buf[1];
      while (isdigit(*p)) {
	p++;
      }
      while (*p++ != '\0') {
	scn_stat.offset -= 2; /* push back the scenario */
      }
      break;
    case 'i':
      cmd->type = FLASH;
      fprintf(stderr, "Flash: i[%s]\n", cmd->buf);
      break;
    case 'm':
      cmd->type = COLOR_MASK;
      fprintf(stderr, "Mask: m[%s]\n", cmd->buf);
      break;
    case 'o':
      cmd->type = FADE_OUT;
      fprintf(stderr, "Fadeout: o[%s]\n", cmd->buf);
      break;
    case 'r':
      cmd->type = PLAY_MUSIC;
      sscanf(cmd->buf, "%d", &(cmd->arg));
      // fprintf(stderr, "Play music: r[%d]\n", cmd->arg);
      break;
    case 't':
      cmd->type = NONE;
      fprintf(stderr, "Unkonwn(color filter?): t[%s]\n", cmd->buf);
      p = cmd->buf;
      while (isdigit(*p) || *p == ',' || *p == '.') {
	tmpbuf[j++] = *p++;
      }
      while (*p++ != '\0') {
	scn_stat.offset -= 2; /* push back the scenario */
      }
      break;
    case 'v':
      cmd->type = NONE;
      fprintf(stderr, "Unkonwn(color filter?): v[%s]\n", cmd->buf);
      p = cmd->buf;
      while (isdigit(*p) || *p == ',') {
	tmpbuf[j++] = *p++;
      }
      while (*p++ != '\0') {
	scn_stat.offset -= 2; /* push back the scenario */
      }
      break;
    case 'w':
      cmd->type = WAIT;
      sscanf(cmd->buf, "%d", &(cmd->arg));
      fprintf(stderr, "wait: w[%d]\n", cmd->arg);
      break;
    case 'z':
      cmd->type = SET_CG;
      fprintf(stderr, "set CG: z[%s]\n", cmd->buf);
      p = cmd->buf;
      while (*p != '/') {
	*p = toupper(*p);
	p++;
      }
      *p++ = '\0';
      cmd->arg = 0;
      j = 0;
      while (*p != '\0') {
	tmpbuf[j++] = *p++;
      }
      tmpbuf[j++] = '\0';
      // fprintf(stderr, "z(set CG): [%s][%s]\n", cmd->buf, tmpbuf);
      break;
    default:
      cmd->type = NONE;
      fprintf(stderr, "Unkonwn: %c[%s]\n", c, cmd->buf);
    }
    return;
  } else { /* UCS-2 KANJI (text) */
    cmd->type = PRINT_TEXT;
    j = 0;
    cmd->ubuf[j++] = ucs;
    for (k = 0; i < script.size[bid]; k++) {
      ucs = (script.block[bid][i + 1] << 8) | script.block[bid][i];
      if (ucs < 0x100) {
	break;
      }
      scn_stat.offset += 2;
      i += 2;
      cmd->ubuf[j++] = ucs;
    }
    cmd->ubuf[j++] = 0;
  }
}

int proc_alternative(int bid)
{
  int i, ucs, idx = 0;

  scn_stat.alt_num = 0;

  /* fprintf(stderr, "bid: %d\n", bid); */
  for (i = 0; i < script.size[bid]; i += 2) {
    ucs = (script.block[bid][i + 1] << 8) | script.block[bid][i];
    if (ucs == 0x0a) {
      scn_stat.alt[scn_stat.alt_num++][idx] = 0;
      if (scn_stat.alt_num > MAX_ALT) {
	fprintf(stderr, "too many alternatives\n");
	quit_proc(top, NULL, NULL);
      }
      idx = 0;
      continue;
    }
    scn_stat.alt[scn_stat.alt_num][idx++] = ucs;
  }  
#if 0
  for (i = 0; i < num; i++) {
    fprintf(stderr, "\n%d:", i + 1);
    puteuc_string(buf[i]);
    fprintf(stderr, "\n");
  }
#endif
  return 0;
}

int next_state(int alt)
{
  scn_stat.offset = 0;
  scn_stat.alt_opening = False;

  if (!strcmp(scn_stat.script, "0x7")) {
    switch(scn_stat.block) {
    case 5:
      return next_block(6);
    case 6:
      if (scn_stat.clear & FLAG_CLEAR_MEGUMI) {
	return next_block_altn(8);
      } else {
	return next_block_altn(7);
      }
    case 7:
      switch (alt) {
      case 0:
	/* Ƥӵ */
	return next_block(9);
      case 1:
	/* 餫Τ */
	return next_block(10);
      default:
	fprintf(stderr, "can't select this alternative.\n");
	quit_proc(top, NULL, NULL);
      }
    case 8:
      switch (alt) {
      case 0:
	/* Ƥӵ */
	return next_block(9);
      case 1:
	/* 餫Τ */
	return next_block(10);
      case 2:
	/* Τǿ³ */
	scn_stat.flag_yue |= YUE1;
	scn_stat.flag_ruria |= RURIA1;
	return next_block(11);
      default:
	fprintf(stderr, "can't select this alternative.\n");
	quit_proc(top, NULL, NULL);
      }
    case 9:
      return next_script("0x1d", 5);
    case 10:
      return next_script("0x1d", 5);
    case 11:
      return next_script("0x1d", 5);
    case 13:
      switch (alt) {
      case 0:
	/* й */
	if (scn_stat.flag_yue & YUE1 ||
	    scn_stat.flag_ruria & RURIA1) {
	}  else {
	  scn_stat.flag_sarasa |= SARASA1;
	  scn_stat.flag_megumi |= MEGUMI1;
	}
	return next_block(14);
      case 1:
	/* ᤰߤٽΤԤ */
	scn_stat.flag_hinata |= HINATA1;
	return next_block(15);
      default:
	fprintf(stderr, "can't select this alternative[%d].\n", alt);
	quit_proc(top, NULL, NULL);
      }
    case 14:
      scn_stat.flag_sarasa |= SARASAa;
      return next_script("0x8", 0);
    case 15:
      return next_script("0x8", 6);
    default:
      fprintf(stderr, "not developped, yet\n");
      break;
    }
  }
  if (!strcmp(scn_stat.script, "0x8")) {
    switch(scn_stat.block) {
    case 0:
      return next_script_altn("0x25", 0);
    case 1:
      return next_block(3);
    case 2:
      return next_block(3);
    case 3:
      if (scn_stat.flag_yue & YUE1) {
	return next_block(4);
      } else {
	return next_block(5); /*  */
      }
    case 4:
      return next_block(7);
    case 5:
      return next_block(7);
    case 6:
      return next_block(7);
    case 7:
      if (scn_stat.flag_yue & YUE1) {
	return next_block(9);
      } else if (scn_stat.flag_sarasa & SARASAa) {
	return next_block(8);
      } else {
	return next_block(14);
      }
    case 8:
      return next_block(14);
    case 9:
      return next_script_altn("0x25", 1);
    case 10:
      return next_block(13);
    case 11:
      return next_block(13);
    case 12:
      return next_block(13);
    case 13:
      return next_script_altn("0x9", 3);
    case 14:
      if (scn_stat.clear &
	  (FLAG_CLEAR_MEGUMI ||
	   FLAG_CLEAR_YUE ||
	   FLAG_CLEAR_RURIA || /* ʤ */
	   FLAG_CLEAR_KURUMI ||
	   FLAG_CLEAR_HIMIKO /* ||
				FLAG_CLEAR_HINATA ||
				FLAG_CLEAR_MINAKO ||
				FLAG_CLEAR_MAO */) &&
	  scn_stat.flag_sarasa & SARASA1) {
	return next_block(15);
      } else {
	return next_script("0x9", 0);
      }
    case 15:
      return next_script_altn("0x25", 2);
    default:
      fprintf(stderr, "not developped, yet\n");
      break;
    }
  }
  if (!strcmp(scn_stat.script, "0x9")) {
    switch(scn_stat.block) {
    case 0:
      return next_block(2);
    case 1:
      return next_block(2);
    case 2:
      return next_block_altn(3);
    case 3:
      switch (alt) {
      case 0:
	/* ᤰߤ˿⻲äˤ */
	return next_block(4);
      case 1:
	/* ˤĤƤҤʤʹ˹Ԥ */
	if (scn_stat.flag_hinata & HINATA1) {
	  scn_stat.flag_hinata |= HINATAb;
	  return next_block(10);
	} else {
	  return next_block(9);
	}
      default:
	fprintf(stderr, "can't select this alternative.\n");
	quit_proc(top, NULL, NULL);
      }
    case 4:
      /* ˥ꥢäѥ 6 ⤢ */
      return next_block(5);
    case 5:
      return next_block(7);
    case 7:
      return next_block(11);
    case 9:
      return next_block(11);
    case 10:
      return next_block(11);
    case 11:
      if (scn_stat.flag_sarasa & SARASA2) {
	return next_block(12);
      } else if (scn_stat.flag_yue & YUE1 || scn_stat.flag_ruria & RURIA1) {
	return next_script("0xa", 1);
      } else {
	return next_script("0xa", 0);
      }
    case 12:
      return next_script_altn("0x25", 3);
    case 14:
      return next_script("0xa", 0);
    default:
      fprintf(stderr, "not developped, yet\n");
      break;
    }
  }
  if (!strcmp(scn_stat.script, "0xa")) {
    switch(scn_stat.block) {
    case 0: /* եå&åץ */
      return next_block(2);
    case 1: /* ɥˡѥ */
      return next_block(2);
    case 2:
      if (scn_stat.flag_yue & YUE1) {
	return next_block(4);
      } else {
	return next_block(3);
      }
    case 3:
      return next_block(5);
    case 4:
      return next_block(5);
    case 5:
      return next_block(6);
    case 6:
      return next_block(7);
    case 7:
      return next_block(8);
    case 8:
      if (scn_stat.clear & FLAG_CLEAR_YUE) {
	if (scn_stat.flag_hinata & HINATA1) {
	  return next_script_altn("0x25", 5);
	} else {
	  return next_script_altn("0x25", 4);
	}
      } else {
	return next_script_altn("0x25", 4);
      }
    case 9:
      if (scn_stat.flag_yue & YUE1) {
	return next_block(11);
      } else {
	return next_block(10);
      }
    case 10:
      return next_block(14);
    case 11:
      return next_script_altn("0x25", 6);
    case 12:
      return next_block(14);
    case 13:
      return next_block(14);
    case 14:
      if (scn_stat.flag_yue & YUE2 ||
	  scn_stat.flag_ruria & RURIA2) {
	return next_script("0xb", 7);
      } else if (scn_stat.flag_sarasa & SARASA3) {
	return next_script("0xb", 6);
      } else {
	return next_script("0xb", 5);
      }
    case 15:
      if (scn_stat.flag_ruria & RURIA1) {
	return next_block(11);
      } else {
	return next_script("0x2a", 5);
      }
    default:
      fprintf(stderr, "not developped, yet\n");
      break;
    }
  }
  if (!strcmp(scn_stat.script, "0xb")) {
    switch(scn_stat.block) {
    case 0:
      return next_script_altn("0x25", 7);
    case 2:
      return next_block(3);
    case 3:
      if (scn_stat.flag_hinata & HINATAb) {
	return next_block(4);
      } else {
	return next_block(5);
      }
    case 4:
      return next_block(8);
    case 5:
      return next_block(8);
    case 6:
      return next_block(8);
    case 7:
      return next_block(8);
    case 8:
      return next_script_altn("0x25", 10);
    case 9:
      return next_block(11);
    case 11:
      if (scn_stat.flag_yue & YUE2 ||
	  scn_stat.flag_ruria & RURIA2) {
	return next_script("0x1d", 13);
      } else if (scn_stat.flag_sarasa & SARASAa) {
	return next_block(13);
      } else {
	return next_block(12);
      }
    case 12:
      return next_block(15);
    case 13:
      return next_block(15);
    case 15:
      return next_script_altn("0x25", 11);
    default:
      fprintf(stderr, "not developped, yet\n");
      break;
    }
  }
  if (!strcmp(scn_stat.script, "0xc")) {
    switch(scn_stat.block) {
    case 0:
      if (scn_stat.flag_yue & YUE1) {
	return next_block(1);
      } else if ((scn_stat.clear & FLAG_CLEAR_MEGUMI) &&
		 (scn_stat.flag_kurumi & KURUMI1 ||
		  scn_stat.flag_hinata & HINATA1)) {
	return next_block(2);
      } else {
	return next_block(1);
      }
    case 1:
      return next_script("0xd", 8);
    case 2:
      return next_script("0xd", 8);
    case 3:
      return next_script_ending("0x1c", 6);
    default:
      fprintf(stderr, "not developped, yet\n");
      break;
    }
  }
  if (!strcmp(scn_stat.script, "0xd")) {
    switch(scn_stat.block) {
    case 8:
      return next_block(9);
    case 9:
      if (scn_stat.flag_sarasa & SARASA3) {
	return next_block(10);
      } else {
	return next_block(12);
      }
    case 10:
      return next_script("0xe", 5);
    case 11:
      return next_script("0x1a", 0);
    case 12:
      if (scn_stat.flag_hinata & HINATAb) {
	return next_block_altn(14);
      } else {
	return next_block_altn(13);
      }
    case 13:
      switch (alt) {
      case 0:
	/* ΢夯ޤǲ̤򤹤 */
	if (scn_stat.flag_kurumi & KURUMI1) {
	  scn_stat.flag_kurumi |= KURUMI2;
	}
	scn_stat.flag_kurumi |= KURUMIa;
	scn_stat.flag_kurumi |= KURUMIb;
	return next_script("0x1d", 15);
      case 1:
	/* ᤰߤݤ뤫Ƥ */
	scn_stat.flag_kurumi |= KURUMIa;
	scn_stat.flag_kurumi |= KURUMIb;
	return next_script("0xe", 2);
      }
    case 14:
      switch (alt) {
      case 0:
	/* ΢夯ޤǲ̤򤹤 */
	scn_stat.flag_kurumi |= KURUMIa;
	scn_stat.flag_kurumi |= KURUMIb;
	return next_script("0x1d", 15);
      case 1:
	/* ᤰߤݤ뤫Ƥ */
	scn_stat.flag_kurumi |= KURUMIa;
	scn_stat.flag_kurumi |= KURUMIb;
	return next_script("0xe", 2);
      case 2:
	/* ر༷Ի׵ġֶ̤ޤä򤹤 */
	return next_script("0xe", 3);
      }
    default:
      fprintf(stderr, "not developped, yet\n");
      break;
    }
  }
  if (!strcmp(scn_stat.script, "0xe")) {
    switch(scn_stat.block) {
    case 0:
      return next_script("0x1e", 1);
    case 1:
      /* 3 ˹Ԥѥ(ߤʹƤʤ) */
      return next_script("0x1e", 1);
    case 2:
      if (scn_stat.flag_kurumi & KURUMI1) {
	return next_block(0);
      } else {
	return next_block(1);
      }
    case 3:
      if (scn_stat.flag_hinata & HINATA1 ||
	  scn_stat.flag_himiko & HIMIKO1) {
	return next_script("0x1e", 3);
      } else {
	return next_block(4);
      }
    case 4:
      return next_block(5);
    case 5:
      return next_script("0x1a", 0);
    case 7:
      return next_script("0x1a", 0);
    case 8:
      return next_script("0x1a", 0);
    case 10:
      return next_block(11);
    case 11:
      return next_script("0x1e", 11);
    case 12:
      return next_script_altn("0x25", 14);
    case 13:
      return next_script("0x1e", 11);
    case 14:
      return next_script("0x1e", 11);
    default:
      fprintf(stderr, "can't select this alternative.\n");
      quit_proc(top, NULL, NULL);
    }
  }
  if (!strcmp(scn_stat.script, "0xf")) {
    switch(scn_stat.block) {
    case 1:
      return next_block(3);
    case 3:
      return next_block(4);
    case 4:
      if (scn_stat.flag_kurumi & KURUMI3) {
	return next_script("0xf", 8);
      } else {
	return next_script("0x1e", 13);
      }
    case 7:
      return next_block(8);
    case 8:
      return next_block(9);
    case 9:
      return next_block(10);
    case 10:
      return next_script_altn("0x26", 0);
    case 11:
      if (scn_stat.flag_kurumi & KURUMI3) {
	return next_block(13);
      } else {
	return next_block(12);
      }
    case 12:
      return next_block(14);
    case 13:
      return next_block(14);
    case 14:
      return next_script("0x10", 2);
    case 15:
      return next_script_ending("0x1c", 6);
    default:
      quit_proc(top, NULL, NULL);
    }
  }
  if (!strcmp(scn_stat.script, "0x10")) {
    switch(scn_stat.block) {
    case 0:
      if (scn_stat.flag_himiko & HIMIKO1) {
	return next_block(1);
      } else {
	return next_block(2);
      }
    case 1:
      return next_block(3);
    case 2:
      return next_block(3);
    case 3:
      if (scn_stat.flag_yue & YUE2 ||
	  scn_stat.flag_ruria & RURIA2) {
	return next_script("0x11", 7);
      } else {
	return next_block(4);
      }
    case 4:
      if (scn_stat.flag_kurumi & KURUMI3) {
	return next_block(5);
      } else {
	return next_block(6);
      }
    case 5:
      return next_block(7);
    case 6:
      return next_block(7);
    case 7:
      return next_script_altn("0x26", 4);
    case 8:
      if (scn_stat.flag_hinata & HINATA1 ||
	  scn_stat.flag_himiko & HIMIKO2 ||
	  scn_stat.flag_mao & MAO1) {
	return next_block(12);
      } else {
	return next_block(9);
      }
    case 9:
      if (scn_stat.flag_kurumi & KURUMI3) {
	return next_block(10);
      } else {
	return next_block(11);
      }
    case 10:
      return next_script("0x11", 12);
    case 11:
      return next_script("0x11", 13);
    case 12:
      return next_script_altn("0x26", 2);
    case 13:
      if (scn_stat.flag_himiko & HIMIKO1) {
	return next_block_altn(15);
      } else {
	return next_block_altn(14);
      }
    case 14:
      switch (alt) {
      case 0:
	/* Ҥʤˤޤ */
	return next_script("0x11", 0);
      case 1:
	/* ᤰߤϡĤʤߤ */
	if (scn_stat.flag_hinata & HINATA3) {
	  scn_stat.flag_hinata |= HINATA4;
	}
	return next_script("0x11", 1);
      }
    case 15:
      switch (alt) {
      case 0:
	/* Ҥʤˤޤ */
	return next_script("0x11", 0);
      case 1:
	/* ᤰߤϡĤʤߤ */
	if (scn_stat.flag_hinata & HINATA3) {
	  scn_stat.flag_hinata |= HINATA4;
	}
	return next_script("0x11", 1);
      case 2:
	/* Ҥʤ򲡤ݤ */
	if (scn_stat.flag_himiko & HIMIKO2) {
	  scn_stat.flag_himiko |= HIMIKO3;
	}
	return next_script("0x11", 2);
      }
    default:
      fprintf(stderr, "can't select this alternative.\n");
      quit_proc(top, NULL, NULL);
    }
  }
  if (!strcmp(scn_stat.script, "0x11")) {
    switch(scn_stat.block) {
    case 0:
      return next_block(4);
    case 1:
      return next_block(6);
    case 2:
      if (scn_stat.flag_himiko & HIMIKO3) {
	return next_block(13);
      } else {
	return next_block(14);
      }
    case 3:
      return next_block(4);
    case 4:
      return next_block(6);
    case 5:
      return next_block(6);
    case 6:
      if (scn_stat.flag_kurumi & KURUMI2) {
	return next_block(12);
      } else {
	return next_block(13);
      }
    case 7:
      return next_script_altn("0x26", 4);
    case 8:
      if (scn_stat.flag_yue & YUE2 ||
	  scn_stat.flag_ruria & RURIA2) {
	return next_script("0x1f", 12);
      } else {
	return next_block(14);
      }
    case 12:
      if (scn_stat.flag_sarasa & SARASA3) {
	return next_script("0x12", 1);
      } else {
	return next_script("0x12", 2);
      }
    case 13:
      return next_script("0x1f", 5);
    case 14:
#if 0
      if (scn_stat.flag_hinata & HINATA1) { /* ???  */}
#endif
      if (0) {
	return next_block(12);
      } else {
	return next_script("0x1f", 5);
      }
    default:
      fprintf(stderr, "can't select this alternative.\n");
      quit_proc(top, NULL, NULL);
    }
  }
  if (!strcmp(scn_stat.script, "0x12")) {
    switch(scn_stat.block) {
    case 1:
      return next_block(3);
    case 2:
      return next_block(3);
    case 3:
      if (scn_stat.flag_sarasa & SARASA3) {
	return next_block(7);
      } else if (scn_stat.flag_kurumi & KURUMI4) {
	return next_block(4);
      } else if (scn_stat.flag_kurumi & KURUMI1) {
	return next_block(6);
      } else if (scn_stat.flag_kurumi & KURUMIb) {
	return next_block(6);
      } else {
	return next_block(5);
      }
    case 4:
      return next_block(14);
    case 5:
      if (scn_stat.flag_kurumi & KURUMI1) {
	return next_block(6);
#if 0
      } else if (scn_stat.flag_himiko & HIMIKO3 ||
		 scn_stat.flag_mao & MAO3) {
	return next_block(11);
#endif
      } else if (scn_stat.flag_himiko & HIMIKO1) {
	return next_block(11);
      } else {
	return next_block(14);
      }
    case 6:
      /* if (scn_stat.flag_himiko & HIMIKO3 || */
      /* scn_stat.flag_mao & MAO1) { */
      /* scn_stat.flag_himiko & HIMIKOa) { */
      if (scn_stat.flag_himiko & HIMIKO1) {
	return next_block(11);
      } else {
	return next_block(14);
      }
    case 7:
      return next_script_altn("0x26", 5);
    case 8:
      return next_script_ending("0x1c", 6);
    case 9:
      return next_script("0x24", 8);
    case 11:
      return next_script_altn("0x26", 6);
    case 12:
      return next_block(15);
    case 14:
      return next_block(15);
    case 15:
      if (scn_stat.flag_himiko & HIMIKO4 ||
	  scn_stat.flag_mao & MAO3) {
	return next_script("0x13", 0);
      } else {
	return next_script("0x13", 1);
      }
    default:
      fprintf(stderr, "can't select this alternative.\n");
      quit_proc(top, NULL, NULL);
    }
  }
  if (!strcmp(scn_stat.script, "0x13")) {
    switch(scn_stat.block) {
    case 0:
      return next_block(2);
    case 1:
      return next_block(2);
    case 2:
      return next_script_altn("0x26", 7);
    case 3:
      return next_block(4);
    case 4:
      return next_script("0x1f", 8);
    case 6:
      return next_script("0x1f", 8);
    case 8:
      if (scn_stat.flag_himiko & HIMIKO4 ||
	  scn_stat.flag_mao & MAO1) {
	return next_script("0x14", 4);
      } else {
	return next_script("0x20", 1);
      }
    case 9:
      return next_script_altn("0x26", 8);
    case 10:
      return next_script("0x20", 1);
    case 13:
      return next_block(15);
    case 14:
      return next_block(15);
    case 15:
      if (scn_stat.flag_ruria & RURIAa) {
	return next_script("0x14", 1);
      } else {
	return next_script("0x1f", 15);
      }
    default:
      fprintf(stderr, "can't select this alternative.\n");
      quit_proc(top, NULL, NULL);
    }
  }
  if (!strcmp(scn_stat.script, "0x14")) {
    switch(scn_stat.block) {
    case 1:
      return next_block(2);
    case 2:
      if (scn_stat.flag_yue & YUE2) {
	return next_script("0x15", 12);
      } else if (scn_stat.flag_ruria & RURIA2) {
	return next_script("0x15", 13);
      } else if (scn_stat.flag_himiko & HIMIKO4 ||
		 scn_stat.flag_mao & MAO1) {
	return next_block(4);
      } else {
	return next_block(-1);
      }
    case 3:
      return next_block(4);
    case 4:
      return next_script("0x20", 1);
    case 6:
      return next_block(8);
    case 7:
      return next_block(8);
    case 8:
      return next_script_altn("0x26", 11);
    case 9:
      return next_block(11);
    case 10:
      return next_block(11);
    case 11:
      if (scn_stat.flag_hinata & HINATA1 ||
	  /* scn_stat.flag_mao & MAO1 || */
	  scn_stat.flag_himiko & HIMIKO1) {
	return next_block(12);
      } else {
	return next_script("0x15", 1);
      }
    case 12:
      return next_script_altn("0x26", 12);
    case 13:
      if (scn_stat.flag_himiko & HIMIKO4 ||
	  scn_stat.flag_mao & MAO4) {
	return next_block(14);
      } else {
	return next_block(15);
      }
    case 14:
      return next_script("0x20", 4);
    case 15:
      return next_script("0x20", 4);
    default:
      fprintf(stderr, "can't select this alternative.\n");
      quit_proc(top, NULL, NULL);
    }
  }
  if (!strcmp(scn_stat.script, "0x15")) {
    switch(scn_stat.block) {
    case 0:
      return next_script("0x20", 4);
    case 1:
      return next_script("0x20", 4);
    case 3:
      return next_script_altn("0x26", 13);
    case 5:
      if (scn_stat.flag_hinata & HINATA5) {
	return next_script("0x21", 14);
      } else {
	return next_script("0x15", 6);
      }
    case 6:
      return next_script("0x20", 6);
    case 8:
      return next_script("0x20", 6);
    case 9:
      return next_script("0x20", 6);
    case 12:
      return next_block(14);
    case 13:
      return next_block(14);
    case 14:
      if (scn_stat.flag_yue & YUE2) {
	return next_block(15);
      } else if (scn_stat.flag_ruria & RURIA2) {
	return next_script("0x16", 0);
      }
    case 15:
      return next_script("0x16", 1);
    default:
      fprintf(stderr, "can't select this alternative.\n");
      quit_proc(top, NULL, NULL);
    }
  }
  if (!strcmp(scn_stat.script, "0x16")) {
    switch(scn_stat.block) {
    case 0:
      return next_block(1);
    case 1:
      return next_script_altn("0x26", 14);
    case 2:
      return next_script_ending("0x1c", 6);
    case 3:
      if (scn_stat.flag_ruria & RURIA2) {
	return next_script("0x20", 9);
      } else {
	return next_block(5);
      }
    case 5:
      return next_script_altn("0x26", 15);
    case 7:
      return next_block(8);
    case 8:
      if (scn_stat.flag_yue & YUE3) {
	return next_block(9);
      } else {
	return next_block(10);
      }
    case 9:
      return next_block(11);
    case 10:
      return next_block(11);
    case 11:
      return next_script_altn("0x27", 0);
    case 12:
      return next_script("0x20", 14);
    case 13:
      return next_script("0x20", 14);
    case 14:
      return next_block(15);
    case 15:
      return next_script("0x21", 9);
    default:
      fprintf(stderr, "can't select this alternative.\n");
      quit_proc(top, NULL, NULL);
    }
  }
  if (!strcmp(scn_stat.script, "0x17")) {
    switch(scn_stat.block) {
    case 1:
      return next_script("0x21", 9);
    case 3:
      return next_block(5);
    case 4:
      return next_script_altn("0x27", 2);
    case 5:
      return next_script_ending("0x1c", 6);
    case 12:
      return next_script_altn("0x27", 3);
    case 13:
      return next_script("0x22", 8);
    default:
      fprintf(stderr, "not developped, yet\n");
      break;
    }
  }
  if (!strcmp(scn_stat.script, "0x18")) {
    switch(scn_stat.block) {
    case 0:
      if (scn_stat.flag_yue & YUE4) {
	return next_block(1);
      } else {
	return next_block(2);
      }
    case 1:
      return next_script_altn("0x27", 4);
    case 2:
      return next_block(4);
    case 4:
      return next_script_ending("0x1c", 6);
    case 5:
      return next_script_altn("0x27", 5);
    case 7:
      return next_block(8);
    case 8:
      if (scn_stat.flag_ruria & RURIA3) {
	return next_block(9);
      } else {
	return next_block(11);
      }
    case 9:
      return next_script_altn("0x27", 6);
    case 10:
      return next_block(12);
    case 11:
      return next_block(14);
    case 12:
      return next_block(13);
    case 13:
      return next_block(15);
    case 14:
      return next_block(15);
    case 15:
      if (scn_stat.flag_ruria & RURIA3) {
	return next_script("0x19", 0);
      } else {
	return next_script("0x19", 1);
      }
    default:
      fprintf(stderr, "not developped, yet\n");
      break;
    }
  }
  if (!strcmp(scn_stat.script, "0x19")) {
    switch(scn_stat.block) {
    case 0:
      return next_block(2);
    case 1:
      return next_block(2);
    case 2:
      if (scn_stat.flag_ruria & RURIA3) {
	return next_script("0x22", 14);
      } else {
	return next_block(3);
      }
    case 3:
      return next_script_ending("0x1c", 6);
    case 6:
      return next_block(7);
    case 7:
      return next_script("0x23", 1);
    case 8:
      return next_script("0x21", 12);
    case 9:
      return next_script_ending("0x1c", 13);
    case 11:
      return next_block(13);
    case 13:
      if (scn_stat.flag_himiko & HIMIKO7) {
	return next_script("0x23", 10);
      } else {
	return next_block(14);
      }
    case 14:
      return next_script_ending("0x1c", 6);
    default:
      fprintf(stderr, "not developped, yet\n");
      break;
    }
  }
  if (!strcmp(scn_stat.script, "0x1a")) {
    switch(scn_stat.block) {
    case 0:
      if (scn_stat.flag_yue & YUE2 ||
	  scn_stat.flag_ruria & RURIA2) {
	return next_block(1);
      } else if (scn_stat.flag_kurumi & KURUMIb) {
	return next_block(2);
      } else {
	return next_block(3);
      }
    case 1:
      return next_script("0x23", 14);
    case 2:
      return next_script("0x23", 14);
    case 3:
      return next_script("0x23", 14);
    case 5:
      return next_block(7);
    case 6:
      return next_block(7);
    case 7:
      if ((scn_stat.clear & FLAG_CLEAR_MEGUMI) &&
	  scn_stat.flag_kurumi & KURUMI2) {
	return next_block(9);
      } else {
	return next_block(8);
      }
    case 8:
      return next_block(12);
    case 9:
      return next_script_altn("0x25", 13);
    case 10:
      return next_block(12);
    case 11:
      return next_block(12);
    case 12:
      if (scn_stat.flag_sarasa & SARASA3) {
	return next_script("0xe", 10);
      }else if (scn_stat.flag_yue & YUE2 ||
		scn_stat.flag_ruria & RURIA2) {
	return next_script("0x1e", 5);
      } else {
	return next_script("0xe", 12);
      }
    case 13:
      return next_script_altn("0x27", 8);
    case 14:
      return next_script("0x1b", 0);
    case 15:
      return next_script("0x1b", 0);
    default:
      fprintf(stderr, "not developped, yet\n");
      break;
    }
  }
  if (!strcmp(scn_stat.script, "0x1b")) {
    switch(scn_stat.block) {
    case 0:
      if (scn_stat.flag_minako & MINAKO1) {
	return next_block(1);
      } else {
	return next_script("0x24", 0);
      }
    case 1:
      return next_script("0x24", 2);
    case 2:
      return next_script("0x1e", 11);
    case 4:
      return next_block(6);
    case 5:
      return next_block(6);
    case 6:
      return next_script_altn("0x27", 9);
    case 7:
      if (scn_stat.flag_minako & MINAKO1) {
	return next_block(8);
      } else {
	return next_script("0x27", 13);
      }
    case 8:
      return next_script_ending("0x1c", 6);
    case 11:
      return next_script_altn("0x27", 10);
    case 12:
      return next_script_ending("0x1c", 6);
    case 15:
      return next_script("0x24", 14);
    }
  }
  if (!strcmp(scn_stat.script, "0x1c")) {
    switch(scn_stat.block) {
    case 2:
      return next_block(6);
    case 3:
      return next_script_altn("0x27", 12);
    case 4:
      return next_block(6);
    case 5:
      return next_block_ending(15);
    case 6:
      /* Хåɥ */
      return goto_title();
    case 7:
      /* ᤰߥ */
      scn_stat.clear |= FLAG_CLEAR_MEGUMI;
      if (scn_stat.clear == FLAG_CLEAR_ALL) {
	return next_script_omake("0x2a", 6);
      }
      return goto_title();
    case 8:
      /* ͥͥ */
      scn_stat.clear |= FLAG_CLEAR_YUE;
      if (scn_stat.clear == FLAG_CLEAR_ALL) {
	return next_script_omake("0x2a", 6);
      }
      return goto_title();
    case 9:
      /* ҥ */
      scn_stat.clear |= FLAG_CLEAR_HIMIKO;
      if (scn_stat.clear == FLAG_CLEAR_ALL) {
	return next_script_omake("0x2a", 6);
      }
      return goto_title();
    case 10:
      /* ߥ */
      scn_stat.clear |= FLAG_CLEAR_KURUMI;
      if (scn_stat.clear == FLAG_CLEAR_ALL) {
	return next_script_omake("0x2a", 6);
      }
      return goto_title();
    case 11:
      /* ҥ */
      scn_stat.clear |= FLAG_CLEAR_MINAKO;
      if (scn_stat.clear == FLAG_CLEAR_ALL) {
	return next_script_omake("0x2a", 6);
      }
      return goto_title();
    case 12:
      /* Ҥʤ */
      scn_stat.clear |= FLAG_CLEAR_HINATA;
      if (scn_stat.clear == FLAG_CLEAR_ALL) {
	return next_script_omake("0x2a", 6);
      }
      return goto_title();
    case 13:
      /* 泌 */
      scn_stat.clear |= FLAG_CLEAR_MAO;
      if (scn_stat.clear == FLAG_CLEAR_ALL) {
	return next_script_omake("0x2a", 6);
      }
      return goto_title();
    case 14:
      /* ꥢ */
      scn_stat.clear |= FLAG_CLEAR_RURIA;
      if (scn_stat.clear == FLAG_CLEAR_ALL) {
	return next_script_omake("0x2a", 6);
      }
      return goto_title();
    case 15:
      /* ӥ */
      scn_stat.clear |= FLAG_CLEAR_SARASA;
      if (scn_stat.clear == FLAG_CLEAR_ALL) {
	return next_script_omake("0x2a", 6);
      }
      return goto_title();
    default:
      fprintf(stderr, "not developped, yet\n");
      break;
    }
  }
  if (!strcmp(scn_stat.script, "0x1d")) {
    switch(scn_stat.block) {
    case 0:
      scn_stat.alt_opening = True;
      music_stop();
      return next_block_altn(1);
    case 1:
      scn_stat.alt_opening = True;
      switch (alt) {
      case 0: /* Ϥ */
	return next_block_altn(2);
      case 1: /* ̩Τ */
	return next_block_altn(3);
      case 2: /* Ͽ򥯥ꥢ */
	scn_stat.clear = 0;
	return next_block_altn(4);
      case 3: /* 򽪤 */
	quit_proc(top, NULL, NULL);
      }
    case 2:
      switch (alt) {
      case 0: /* ǽ餫 */
	return next_script("0x7", 5);
      case 1: /* ³ */
	load_dialog();
	scn_stat.alt_opening = True;
	return next_block_altn(2);
      case 2: /*  */
	scn_stat.alt_opening = True;
	return next_block_altn(1);
      }
    case 3:
      switch (alt) {
      case 0:  /* ҡ̩Х */
	scn_stat.alt_opening = True;
	return next_block_altn(1);
      case 1: /*  */
	scn_stat.alt_opening = True;
	return next_block_altn(1);
      case 2: /*  */
	scn_stat.alt_opening = True;
	return next_block_altn(1);
      }
    case 4:
      switch (alt) {
      case 0:  /* Ͽ򥯥ꥢ */
	fprintf(stderr, "*\n");
	fprintf(stderr, "* All records are cleared.\n");
	fprintf(stderr, "*\n");
	scn_stat.alt_opening = True;
	return next_block_altn(1);
      case 1:  /*  */
	scn_stat.alt_opening = True;
	return next_block_altn(1);
      }
    case 5:
      return next_block(6);
    case 6:
      if (scn_stat.clear & FLAG_CLEAR_YUE) {
	return next_script_altn("0x7", 13);
      } else {
	return next_script("0x7", 14);
      }
    case 7:
      return next_block(8);
    case 8:
      return next_script("0x9", 14);
    case 9:
      return next_block(10);
    case 10:
      return next_script("0xb", 4);
    case 11:
      return next_block(12);
    case 12:
      return next_script("0xb", 11);
    case 13:
      return next_block(14);
    case 14:
      return next_script("0xd", 11);
    case 15:
      return next_script("0x1e", 0);
    default:
      fprintf(stderr, "can't select this alternative.\n");
      quit_proc(top, NULL, NULL);
    }
  }
  if (!strcmp(scn_stat.script, "0x1e")) {
    switch(scn_stat.block) {
    case 0:
      if (scn_stat.flag_kurumi & KURUMI1) {
	return next_script("0xe", 0);
      } else {
	return next_script("0xe", 1);
      }
    case 1:
      return next_block(2);
    case 2:
      return next_script_altn("0x25", 12);
    case 3:
      return next_block(4);
    case 4:
      return next_script("0x1a", 0);
    case 5:
      return next_block(6);
    case 6:
      return next_script("0x1b", 2);
    case 11:
      if (scn_stat.flag_kurumi & KURUMI2) {
	return next_script("0x29", 10);
      } else {
	return next_script("0x29", 11);
      }
    case 12:
      if (scn_stat.flag_yue & YUE2 ||
	  scn_stat.flag_ruria & RURIA2) {
	return next_script("0xf", 7);
      } else {
	return next_script("0xf", 1);
      }
    case 13:
      return next_block(14);
    case 14:
      return next_script_altn("0x25", 15);
    case 15:
      return next_script("0x1f", 0);
    default:
      fprintf(stderr, "can't select this alternative.\n");
      quit_proc(top, NULL, NULL);
    }
  }
  if (!strcmp(scn_stat.script, "0x1f")) {
    switch(scn_stat.block) {
    case 0:
      return next_script("0x10", 0);
    case 1:
      return next_block(2);
    case 2:
      return next_script_altn("0x26", 3);
    case 3:
      return next_block(4);
    case 4:
      return next_block(12);
    case 5:
      return next_block(6);
    case 6:
      return next_block(7);
    case 7:
      if (scn_stat.flag_sarasa & SARASA3) {
	return next_script("0x12", 1);
      } else {
	return next_script("0x12", 2);
      }
    case 8:
      return next_block(9);
    case 9:
      if (scn_stat.flag_kurumi & KURUMI4) {
	return next_script("0x13", 9);
#if 0
	/* ļԤäƤƤ⤯ߥʥꥪʤ郎褦ʡ */
      } else if (scn_stat.flag_kurumi & KURUMI3) {
	return next_script("0x13", 9);
#endif	  
      } else {
	return next_script("0x13", 8);
      }
    case 10:
      return next_block(11);
    case 11:
      return next_script("0x1a", 13);
    case 12:
      /* ΥץȤ̷⤢(ĤƤʤΤȤˤʤäƤ) */
      return next_block(13);
    case 13:
      return next_block(14);
    case 14:
      return next_script_altn("0x26", 9);
    case 15:
      return next_script("0x20", 0);
    default:
      fprintf(stderr, "can't select this alternative.\n");
      quit_proc(top, NULL, NULL);
    }
  }
  if (!strcmp(scn_stat.script, "0x20")) {
    switch(scn_stat.block) {
    case 0:
      return next_script("0x14", 2);
    case 1:
      return next_block(2);
    case 2:
      return next_block(3);
    case 3:
      if (scn_stat.flag_hinata & HINATA1 ||
	  scn_stat.flag_mao & MAO1 ||
	  scn_stat.flag_himiko & HIMIKO1) {
	return next_script("0x14", 7);
      } else {
	return next_script_altn("0x26", 10);
      }
    case 4:
      return next_block(5);
    case 5:
#if 0
      if (scn_stat.flag_hinata & HINATA3 ||
	  scn_stat.flag_mao & MAO3 ||
	  scn_stat.flag_himiko & HIMIKO4) {
      }
#endif
      if (scn_stat.flag_himiko & HIMIKOc) {
	return next_script("0x15", 3);
      } else {
	return next_script("0x15", 9);
      }
    case 6:
      return next_block(7);
    case 7:
      return next_block(8);
    case 8:
      return next_script("0x21", 0);
    case 9:
      return next_block(10);
    case 10:
      return next_block(11);
    case 11:
      return next_script("0x18", 5);
    case 12:
      return next_block(13);
    case 13:
      return next_script("0x16", 8);
    case 14:
      return next_block(15);
    case 15:
      if (scn_stat.flag_yue & YUE3) {
	return next_script("0x17", 12);
      } else {
	return next_script("0x17", 13);
      }
    default:
      fprintf(stderr, "can't select this alternative.\n");
      quit_proc(top, NULL, NULL);
    }
  }
  if (!strcmp(scn_stat.script, "0x21")) {
    switch(scn_stat.block) {
    case 0:
      return next_block(1);
    case 1:
      return next_script("0x2a", 1);
    case 3:
      return next_script("0x29", 13);
    case 5:
      return next_block(6);
    case 6:
      return next_block(7);
    case 7:
      return next_script_altn("0x27", 1);
    case 9:
      return next_block(10);
    case 10:
      if (scn_stat.flag_megumi & MEGUMI3) {
	return next_script("0x17", 4);
      } else {
	return next_script("0x17", 3);
      }
    case 11:
      return next_script("0x19", 8);
    case 12:
      return next_block(13);
    case 13:
      return next_script_ending("0x1c", 7);
    case 14:
      return next_block(15);
    case 15:
      return next_script("0x22", 0);
    default:
      fprintf(stderr, "can't select this alternative.\n");
      quit_proc(top, NULL, NULL);
    }
  }
  if (!strcmp(scn_stat.script, "0x22")) {
    switch(scn_stat.block) {
    case 0:
      return next_block(1);
    case 1:
      return next_block(2);
    case 2:
      return next_block(3);
    case 3:
      return next_block(4);
    case 4:
      return next_script_ending("0x1c", 12);
    case 5:
      return next_block(6);
    case 6:
      return next_block(7);
    case 7:
      return next_script("0x18", 0);
    case 8:
      return next_block(9);
    case 9:
      return next_script("0x18", 0);
    case 10:
      return next_block(11);
    case 11:
      return next_script_ending("0x1c", 8);
    case 12:
      return next_block(13);
    case 13:
      return next_script("0x18", 8);
    case 14:
      return next_block(15);
    case 15:
      return next_script("0x23", 0);
    default:
      fprintf(stderr, "not developped, yet\n");
      break;
    }
  }
  if (!strcmp(scn_stat.script, "0x23")) {
    switch(scn_stat.block) {
    case 0:
      return next_script_ending("0x1c", 14);
    case 1:
      return next_block(2);
    case 2:
      return next_script("0x19", 9);
    case 3:
      return next_block(4);
    case 4:
      return next_script_altn("0x27", 7);
    case 5:
      return next_block(6);
    case 6:
      return next_block(7);
    case 7:
      return next_block(8);
    case 8:
      return next_block(9);
    case 9:
      return next_script("0x19", 13);
    case 10:
      return next_block(11);
    case 11:
      return next_block(12);
    case 12:
      return next_block(13);
    case 13:
      return next_script_ending("0x1c", 9);
    case 14:
      return next_block(15);
    case 15:
      if (scn_stat.flag_kurumi & KURUMIa) {
	return next_script("0x29", 5);
      } else {
	return next_script("0x29", 6);
      }
    default:
      fprintf(stderr, "can't select this alternative.\n");
      quit_proc(top, NULL, NULL);
    }
  }
  if (!strcmp(scn_stat.script, "0x24")) {
    switch(scn_stat.block) {
    case 0:
      return next_block(1);
    case 1:
      return next_block(2);
    case 2:
      return next_block(3);
    case 3:
      if (scn_stat.flag_minako & MINAKO1) {
	return next_script("0x1b", 5);
      } else {
	return next_script("0x1b", 4);
      }
    case 4:
      return next_block(5);
    case 5:
      return next_script_ending("0x1c", 10);
    case 6:
      return next_block(7);
    case 7:
      return next_script_ending("0x1c", 11);
    case 8:
      return next_block(9);
    case 9:
      return next_block(10);
    case 10:
      return next_block(11);
    case 11:
      return next_script_altn("0x27", 11);
    case 12:
      return next_block(13);
    case 13:
      return next_block(14);
    case 14:
      return next_block(15);
    case 15:
      if (scn_stat.flag_sarasa & SARASA4) {
	return next_script("0x1c", 3);
      } else {
	return next_script("0x1c", 2);
      }
    }
  }
  if (!strcmp(scn_stat.script, "0x25")) {
    switch(scn_stat.block) {
    case 0:
      switch (alt) {
      case 0:
	/* 虜ȤäƤ */
	return next_script("0x8", 1); 
      case 1:
	/* ŷܤ */
	return next_script("0x8", 2);
      }
    case 1:
      switch (alt) {
      case 0:
	/* εä */
	return next_script("0x8", 10); 
      case 1:
	/* ФäƤ */
	return next_script("0x8", 11);
      case 2:
	/* ƤФä */
	return next_script("0x8", 12);
      }
    case 2:
      switch (alt) {
      case 0:
	/* ;פʥȥ쥹Ϳʤ */
	return next_script("0x9", 0);
      case 1:
	/* ͤ䤤ͤ */
	if (scn_stat.flag_sarasa & SARASA1) {
	  scn_stat.flag_sarasa |= SARASA2;
	}
	return next_script("0x9", 1);
      }
    case 3:
      switch (alt) {
      case 0:
	/* ޤǲظ */
	if (scn_stat.flag_sarasa & SARASA2) {
	  scn_stat.flag_sarasa |= SARASA3;
	}
	return next_script("0x1d", 7); 
      case 1:
	/* ޤ */
	return next_script("0x9", 14);
      }
    case 4:
      switch (alt) {
      case 0:
	/* ǯζ̹ˤعԤ */
	return next_script("0xa", 9);
      case 1:
	/* ̲Τݷ򼼤عԤ */
	scn_stat.flag_kurumi |= KURUMI1;
	scn_stat.flag_kurumi |= KURUMIa;
	return next_script("0xa", 15);
      }
    case 5:
      switch (alt) {
      case 0:
	/* ǯζ̹ˤعԤ */
	return next_script("0xa", 9);
      case 1:
	/* ̲Τݷ򼼤عԤ */
	scn_stat.flag_kurumi |= KURUMI1;
	scn_stat.flag_kurumi |= KURUMIa;
	return next_script("0xa", 15);
      case 2:
	/* ҤιʲعԤ */
	scn_stat.flag_himiko |= HIMIKOc;
	return next_script("0xb", 0);
      }
    case 6:
      switch (alt) {
      case 0:
	/* ͤλ೼äƤ */
	scn_stat.flag_ruria |= RURIA2;
	return next_script("0xa", 12);
      case 1:
	/* ͭΥǡȥݥå */
	scn_stat.flag_yue |= YUE2;
	return next_script("0xa", 13);
      }
    case 7:
      switch (alt) {
      case 0:
	/* Τޤ */
	return next_script("0x1d", 9);
      case 1:
	/* ٤ */
	if (scn_stat.flag_hinata & HINATA1) {
	  scn_stat.flag_himiko |= HIMIKO1;
	  scn_stat.flag_mao |= MAO1;
	}
	return next_script("0xb", 2);
      }
    case 8:
      return next_script("0xb", 5);
    case 10:
      switch (alt) {
      case 0:
	/* ۤäƺǸޤʹ */
	return next_script("0xb", 9);
      case 1:
	/* ʹʤ */
	if (scn_stat.flag_himiko & HIMIKO1) {
	  scn_stat.flag_himiko |= HIMIKO2;
	}
	scn_stat.flag_himiko |= HIMIKOa;
	return next_script("0x1d", 11);
      }
    case 11:
      switch (alt) {
      case 0:
	/* ҤߤǤʤ */
	return next_script("0xc", 0);
      case 1:
	/* ̵Ǥ˰櫓ˤϤʤ */
	return next_script("0xc", 3);
      }
    case 12:
      switch (alt) {
      case 0:
	/* ᤰߤ« */
	return next_script("0xe", 7);
      case 1:
	/* ʹƤ */
	return next_script("0xe", 8);
      }
    case 13:
      switch (alt) {
      case 0:
	/* ڤ򸫤˹Ԥ */
	if(scn_stat.flag_kurumi & KURUMI2) {
	  scn_stat.flag_kurumi |= KURUMI3;
	}
	return next_script("0x1a", 10);
      case 1:
	/* ᤰߤԤäƤΤǤ */
	return next_script("0x1a", 11);
      }
    case 14:
      switch (alt) {
      case 0:
	/* ˻ųݤ */
	return next_script("0xe", 13);
      case 1:
	/* ľ˵ */
	return next_script("0xe", 14);
      }
    case 15:
      switch (alt) {
      case 0:
	/* ҤϢ */
	return next_script("0x1e", 15);
      case 1:
	/* ٻϢ */
	return next_script("0xf", 15);
      }
    default:
      fprintf(stderr, "not developped, yet\n");
      break;
    }
  }
  if (!strcmp(scn_stat.script, "0x26")) {
    switch(scn_stat.block) {
    case 0:
      switch (alt) {
      case 0:
	/* ҤϢ */
	return next_script("0xf", 11);
      case 1:
	/* ٻϢ */
	return next_script("0xf", 15);
      }
    case 2:
      switch (alt) {
      case 0:
	/* 괺Ҥʤ򤫤餫 */
	return next_script("0x1f", 1);
      case 1:
	/* Ҥʤ̵뤷ƲĤƤΩʹ */
	return next_script("0x11", 5);
      }
    case 3:
      switch (alt) {
      case 0:
	/* Ҥʤ˥ */
	if (scn_stat.flag_hinata & HINATA1) {
	  scn_stat.flag_hinata |= HINATA3;
	} else if (scn_stat.flag_himiko & HIMIKO2) {
	  scn_stat.flag_himiko |= HIMIKO3;
	}
	return next_script("0x10", 13);
      case 1:
	/* ҤʤΥ */
	return next_script("0x11", 3);
      }
    case 4:
      switch (alt) {
      case 0:
	/* ļͻҤ򸫤˹Ԥ */
	if (scn_stat.flag_kurumi & KURUMI3) {
	  scn_stat.flag_kurumi |= KURUMI4;
	}
	if (scn_stat.flag_yue & YUE2 ||
	    scn_stat.flag_ruria & RURIA2) {
	  return next_script("0x1f", 3);
	} else if (scn_stat.flag_kurumi & KURUMI3) {
	  return next_script("0x10", 8);
	} else {
	  return next_script("0x10", 8);
	}
      case 1:
	/* ȤʤԤäƤ */
	scn_stat.flag_hinata |= HINATAa;
	return next_script("0x11", 8);
      }
    case 5:
      switch (alt) {
      case 0:
	/* ӤȰ˹Ԥ */
	return next_script("0x12", 8);
      case 1:
	/* ˹Ԥ櫓ˤϤʤ */
	return next_script("0x12", 9);
      }
    case 6:
      switch (alt) {
      case 0:
	/* ˿λä */
	if (scn_stat.flag_himiko & HIMIKO3) {
	  scn_stat.flag_himiko |= HIMIKO4;
	}
	if (scn_stat.flag_mao & MAO1) {
	  scn_stat.flag_mao |= MAO3;
	}
	return next_script("0x12", 12);
      case 1:
	/* ޤо夵˲񤤤˹Ԥ*/
	return next_script("0x12", 15);
      }
    case 7:
      switch (alt) {
      case 0:
	/* ˤᤰߤϤޤ±عԤ */
	scn_stat.flag_megumi |= MEGUMI2;
	return next_script("0x13", 3);
      case 1:
	/* ˸ذ⤹ */
	return next_script("0x13", 6);
      }
    case 8:
      switch (alt) {
      case 0:
	/* ݤΤǡäƤ */
	return next_script("0x13", 10);
      case 1:
	/* ͻҤ򸫤˸äƤ */
	if (scn_stat.flag_kurumi & KURUMI3) {
	  scn_stat.flag_kurumi |= KURUMI5;
	}
	return next_script("0x1f", 10);
      }
    case 9:
      switch (alt) {
      case 0:
	/* ᤰߤޤ˹Ԥ */
	return next_script("0x13", 13);
      case 1:
	/* ꥢޤ˹Ԥ */
	scn_stat.flag_ruria |= RURIAa;
	return next_script("0x13", 14);
      }
    case 10:
      switch (alt) {
      case 0:
	/* ¸ */
	return next_script("0x14", 6);
      case 1:
	/* ڡϻߤƤ */
	return next_script("0x14", 7);
      }
    case 11:
      switch (alt) {
      case 0:
	/* ޤʤʤ */
	return next_script("0x14", 9);
      case 1:
	/* ˲񤤤褿˷ޤäƤʡ */
	return next_script("0x14", 10);
      }
    case 12:
      switch (alt) {
      case 0:
	/* ԿۤޤƤ롪 */
	if (scn_stat.flag_himiko & HIMIKO5) {
	  scn_stat.flag_himiko |= HIMIKO6;
	}
	if (scn_stat.flag_mao & MAO3) {
	  scn_stat.flag_mao |= MAO4;
	}
	if (scn_stat.flag_hinata & HINATA4) {
	  scn_stat.flag_hinata |= HINATA5;
	}
	return next_script("0x14", 13);
      case 1:
	/* ѤۤˤϴؤʤΤ֤ */
	return next_script("0x15", 0);
      }
    case 13:
      switch (alt) {
      case 0:
	/* 狼ä˹Ԥ */
	return next_script("0x15", 6);
      case 1:
	/* ˡäȥȥˤ */
	return next_script("0x15", 5);
      case 2:
	/* ޽񼼤֤ܤäΤ˺Ƥ */
	if (scn_stat.flag_himiko & HIMIKO4) {
	  scn_stat.flag_himiko |= HIMIKO5;
	  return next_script("0x23", 3);
	} else if (scn_stat.flag_mao & MAO4) {
	  return next_script("0x19", 6);
	} else {
	  return next_script("0x15", 8);
	}
      }
    case 14:
      switch (alt) {
      case 0:
	/* ߵ޼֤Ƥ */
	return next_script("0x16", 2);
      case 1:
	/* ҤϢ */
	return next_script("0x16", 3);
      }
    case 15:
      switch (alt) {
      case 0:
	/* ͥͤߤ */
	scn_stat.flag_yue |= YUE3;
	return next_script("0x20", 12);
      case 1:
	/* ! ϤǤʤ! */
	return next_script("0x16", 7);
      }
    }
  }
  if (!strcmp(scn_stat.script, "0x27")) {
    switch(scn_stat.block) {
    case 0:
      switch (alt) {
      case 0:
	/* ں */
	return next_script("0x16", 12);
      case 1:
	/* ŰǺغ */
	return next_script("0x16", 13);
      }
    case 1:
      switch (alt) {
      case 0:
	/* ᤰߤ */
	if (scn_stat.flag_megumi & MEGUMI2) {
	  scn_stat.flag_megumi |= MEGUMI3;
	}
	return next_script("0x16", 14);
      case 1:
	/* ñ夵 */
	return next_script("0x17", 1);
      }
    case 2:
      switch (alt) {
      case 0:
	/* ̼Ͽ */
	return next_script("0x17", 5);
      case 1:
	/* 󤸤ʤ */
	return next_script("0x21", 11);
      }
    case 3:
      switch (alt) {
      case 0:
	/* ͥͤδ¤򤹤 */
	scn_stat.flag_yue |= YUE4;
	return next_script("0x22", 5);
      case 1:
	/* Ҥμ򤹤 */
	return next_script("0x22", 8);
      }
    case 4:
      switch (alt) {
      case 0:
	/* ʪ˿ΤϤ */
	return next_script("0x22", 10);
      case 1:
	/* Ϥʤ */
	return next_script("0x18", 4);
      }
    case 5:
      switch (alt) {
      case 0:
	/* ꥢܤˤͳʤ */
	scn_stat.flag_ruria |= RURIA3;
	return next_script("0x22", 12);
      case 1:
	/* ͥͤϤɤ */
	return next_script("0x18", 7);
      }
    case 6:
      switch (alt) {
      case 0:
	/* ҤΤ餻 */
	return next_script("0x18", 10);
      case 1:
	/* ꥢȿڤعԤ */
	return next_script("0x18", 11);
      }
    case 7:
      switch (alt) {
      case 0:
	/*  ޤä */
	return next_script("0x19", 11);
      case 1:
	/* ŰǺ˴ */
	scn_stat.flag_himiko |= HIMIKO7;
	return next_script("0x23", 5);
      }
    case 8:
      switch (alt) {
      case 0:
	/* ζκڤ */
	if(scn_stat.flag_kurumi & KURUMI5) {
	  scn_stat.flag_kurumi |= KURUMI6;
	}
	return next_script("0x1a", 14);
      case 1:
	/*  */
	scn_stat.flag_minako |= MINAKO1;
	return next_script("0x1a", 15);
      }
    case 9:
      switch (alt) {
      case 0:
	/* о夵ޤԤȤ */
	return next_script("0x1b", 7);
      case 1:
	/* عƧ߹ */
	if (scn_stat.flag_minako & MINAKO1) {
	  return next_script("0x1b", 11);
	} else {
	  return next_script("0x24", 4);
	}
      }
    case 10:
      switch (alt) {
      case 0:
	/*  */
	return next_script("0x1b", 12);
      case 1:
	/*  */
	return next_script("0x24", 6);
      }
    case 11:
      switch (alt) {
      case 0:
	/*  */
	return next_script("0x1b", 15);
      case 1:
	/* Ф˷ */
	scn_stat.flag_sarasa |= SARASA4;
	return next_script("0x24", 12);
      }
    case 12:
      switch (alt) {
      case 0:
	/* Ҥ˹ӤϤ */
	return next_script("0x1c", 4);
      case 1:
	/* ӤȰ˹Ԥ */
	return next_script("0x1c", 5);
      }
    case 13:
      return next_script_ending("0x1c", 6);
    }
  }
  if (!strcmp(scn_stat.script, "0x28")) {
  }
  if (!strcmp(scn_stat.script, "0x29")) {
    switch(scn_stat.block) {
    case 5:
      return next_block(7);
    case 6:
      return next_block(7);
    case 7:
      if (scn_stat.flag_kurumi & KURUMI2) {
	return next_script("0x1a", 5);
      } else {
	return next_script("0x1a", 6);
      }
    case 10:
      return next_block(12);
    case 11:
      return next_block(12);
    case 12:
      return next_script("0x1e", 12);
    case 13:
      if (scn_stat.flag_megumi & MEGUMI2) {
	return next_block(14);
      } else {
	return next_script("0x2a", 0);
      }
    case 14:
      return next_block(15);
    case 15:
      return next_script("0x21", 5);
    }
  }
  if (!strcmp(scn_stat.script, "0x2a")) {
    switch(scn_stat.block) {
    case 0:
      return next_script("0x21", 5);
    case 1:
      return next_block(2);
    case 2:
      return next_block(4);
    case 3: /* 2, 3 ʬ */
      return next_block(4);
    case 4:
      return next_script("0x21", 3);
    case 5:
      return next_script("0x25", 8);
    case 6:
      return next_block_altn(7);
    case 7:
      switch (alt) {
      case 0: /* ե¸ */
	output_omake();
	break;
      case 1: /* ե¸ʤ */
	/* do nothing */
	break;
      }
      return goto_title();
    default:
      fprintf(stderr, "can't select this alternative.\n");
      quit_proc(top, NULL, NULL);
    }
  }
  fprintf(stderr, "Unknown script\n");
  return STAT_WAIT;
}

static int next_block(int block)
{
  scn_stat.block = block;
  return STAT_PROC;
}

static int next_block_altn(int block)
{
  scn_stat.block = block;
  return STAT_ALTN;
}

static int next_block_ending(int block)
{
  scn_stat.block = block;
  return STAT_ENDING;
}

static int next_script(const char *script, int block)
{
  free_script();
  strcpy(scn_stat.script, script);
  scn_stat.block = block;
  scn_stat.init_state = STAT_PROC;
  return STAT_INIT;
}

static int next_script_altn(const char *script, int block)
{
  free_script();
  strcpy(scn_stat.script, script);
  scn_stat.block = block;
  scn_stat.init_state = STAT_ALTN;
  return STAT_INIT;
}

static int next_script_omake(const char *script, int block)
{
  free_script();
  strcpy(scn_stat.script, script);
  scn_stat.block = block;
  scn_stat.init_state = STAT_OMAKE;
  return STAT_INIT;
}

static int next_script_offset(const char *script, int block, int offset)
{
  free_script();
  strcpy(scn_stat.script, script);
  scn_stat.block = block;
  scn_stat.offset = offset;
  scn_stat.init_state = STAT_PROC;
  return STAT_INIT;
}

static int next_script_ending(const char *script, int block)
{
  free_script();
  strcpy(scn_stat.script, script);
  scn_stat.block = block;
  scn_stat.init_state = STAT_ENDING;
  return STAT_INIT;
}

static int goto_title()
{
  init_scenario();

  next_script_offset("0x1d", 0, TITLE_OFFSET);
  
  scn_stat.alt_opening = True;
  scn_stat.init_state = STAT_OPENING;

  return STAT_INIT;
}
